"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var express_1 = __importDefault(require("express"));
var app = express_1.default();
var XXEServer = /** @class */ (function () {
    function XXEServer(address, port) {
        if (address === void 0) { address = "127.0.0.1"; }
        if (port === void 0) { port = 7777; }
        this.address = address;
        this.port = port;
        this.started = false;
    }
    XXEServer.prototype.getServerUrl = function () {
        return "http://" + this.address + ":" + this.port;
    };
    XXEServer.prototype.addRoute = function (route, responseContent, printQueryString) {
        if (printQueryString === void 0) { printQueryString = false; }
        if (process.env.NODE_ENV !== "test")
            console.log("Setting route " + route);
        app.get(route, function (request, response) {
            if (process.env.NODE_ENV !== "test") {
                console.log("-----------------------------------------------------------");
                console.log("[" + new Date().toISOString() + "] - Serving route " + route + " to ip " + request.ip);
            }
            if (printQueryString)
                console.log(request.query);
            response.send(responseContent);
        });
        return this.getServerUrl() + route;
    };
    XXEServer.prototype.addExtractionRoute = function () {
        this.addRoute("/", "", true);
    };
    XXEServer.prototype.getServer = function () {
        return app;
    };
    XXEServer.prototype.start = function () {
        if (!this.started) {
            if (process.env.NODE_ENV !== "test") {
                console.log("Starting server at " + this.getServerUrl());
                console.log("-----------------------------------------------------------");
            }
            this.server = app.listen(this.port, this.address);
            this.started = true;
        }
    };
    XXEServer.prototype.stop = function () {
        var _a;
        if (process.env.NODE_ENV !== "test")
            console.log("Stopping server");
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.close();
        this.started = false;
    };
    return XXEServer;
}());
exports.XXEServer = XXEServer;
